#include<iostream>
#include<cstdio>
#include<vector>

using namespace std;
typedef vector<int> bignum;
const int base = 1e4;

void print(bignum a){
    printf("%d", a[a.size()-1]);
    for(int i = a.size()-2; i >= 0; --i){
        printf("%0.4d", a[i]);
    }
    printf("\n");
}

void norm(bignum& a){
    int carry = 0;
    for(int i = 0; i < a.size() || carry; ++i){
        if(a.size() == i) a.push_back(0);
        int num = a[i] + carry;
        a[i] = num % base;
        carry = num / base;
    }
}

bignum& operator+=(bignum& a, const bignum& b){
    for(int i = 0; i < b.size(); ++i){
        if(a.size() == i) a.push_back(0);
        a[i] += b[i];
    }
    norm(a);
    return a;
}

bignum operator+(bignum a, const bignum& b){
    return (a += b);
}

bignum operator*(const bignum& a, const bignum& b){
    bignum result(a.size() + b.size() - 1, 0);
    for(int i = 0; i < a.size(); ++i)
    for(int j = 0; j < b.size(); ++j)
        result[i+j] += a[i] * b[j];
    norm(result);
    return result;
}

bignum& operator*=(bignum& a, int b){
    for(int i = 0; i < a.size(); ++i)
        a[i] *= b;
    norm(a);
    return a;
}

bignum pow(int a, int b){
    bignum num(1, 1);
    bignum power(1, a);
    norm(power);
    while(b){
        if(b & 1)
            num = num * power;
        power = power * power;
        b >>= 1;
    }
    return num;
}

vector<bignum> f(1, bignum(1, 1));
bignum& fact(int n){
    n >>= 1;
    for(int i = f.size(); i <= n; ++i){
        f.push_back(f.back());
        f[i] *= 2*i+1;
    }
    return f[n];
}

bignum Cnk[101][101];
bignum& C(int n, int k){
    if(Cnk[n][k].empty()){
        if(n == k || k == 0)
            Cnk[n][k] = bignum(1, 1);
        else
            Cnk[n][k] = C(n-1, k) + C(n-1, k-1);
    }
    return Cnk[n][k];
}

int n;
int permutation[100];
int cycles[101];

void cycles_count(){
    for(int i = 0; i < n; ++i){
        if(permutation[i] == -1) continue;
        int pos = i;
        int len = 0;
        while(permutation[pos] != -1){
            int to = permutation[pos];
            permutation[pos] = -1;
            pos = to;
            ++len;
        }
        ++cycles[len];
    }
}

bignum solve(){
    bignum ans(1, 1);
    for(int i = 1; i <= n; ++i){
        int N = cycles[i];
        if(N == 0) continue;
        bignum num(1, 1);
        if(i & 1)
            for(int j = 1; j <= N/2; ++j)
                num += pow(i, j) * C(N, 2*j) * fact(2*j-1);
        else{
            if(N & 1) return bignum(1, 0);
            num = pow(i, N/2) * fact(N-1);
        }
        ans = ans * num;
    }
    return ans;
}

int main(){
    freopen("sleep2.in", "r", stdin);
    freopen("sleep2.out", "w", stdout);
    scanf("%d", &n);
    for(int i = 0; i < n; ++i){
        scanf("%d", &permutation[i]);
        --permutation[i];
    }
    cycles_count();
    print(solve());
}
